import { Local } from "@/utils/erpStorage.js";
import {debounce} from 'lodash-es'
import { setupDirective as setupCustomDirectives } from '@/directive/index.ts'
// 缓存按钮权限列表的key
const BTN_PERMISSION_KEY = 'ruleBtn';

// 工具函数: 获取权限列表
const getPermissionList = () => Local.get(BTN_PERMISSION_KEY) || [];

// 工具函数: 安全地移除DOM元素
const safeRemoveElement = (el) => {
  el.style.display = 'none';
  requestAnimationFrame(() => el.parentNode?.removeChild(el));
};

export default (app) => {
  // 防抖函数
  const directives = {
    // 禁止输入空格
    'no-space': {
      mounted(el) {
        const input = el.tagName === 'INPUT' ? el : el.querySelector('input');
        if (!input) return;
        const handleInput = debounce((e) => {
          const value = e.target.value;
          const trimmedValue = value.replace(/\s/g, '');
          if (value !== trimmedValue) {
            e.target.value = trimmedValue;
            // 创建一次性的input事件
            const inputEvent = new InputEvent('input', { bubbles: true });
            e.target.dispatchEvent(inputEvent);
          }
        }, 100);

        input.addEventListener('input', handleInput);

        // 存储清理函数
        el._cleanup = () => input.removeEventListener('input', handleInput);
      },
      unmounted(el) {
        // 清理事件监听
        el._cleanup?.();
      }
    },

    // 上拉加载更多
    'loadmore': {
      mounted(el, binding) {
        const handleScroll = debounce(() => {
          const { scrollHeight, scrollTop, clientHeight } = el;
          const threshold = 50; // 提前触发的距离
          const isNearBottom = scrollHeight - scrollTop - clientHeight <= threshold;

          if (isNearBottom && typeof binding.value === 'function') {
            binding.value();
          }
        }, 200);

        el.addEventListener('scroll', handleScroll, { passive: true });

        el._cleanup = () => el.removeEventListener('scroll', handleScroll);
      },
      unmounted(el) {
        el._cleanup?.();
      }
    },

    // 按钮权限控制
    'permission': {
      mounted(el, binding) {
        if (!binding.value) return;
        const btnList = getPermissionList();
        const hasPermission = btnList.includes(binding.value);

        if (!hasPermission) {
          safeRemoveElement(el);
        }
      }
    }
  };

  // 注册指令
  Object.entries(directives).forEach(([name, directive]) => {
    app.directive(name, directive);
  });

  // 权限检查函数
  const hasPermission = (permissions) => {
    const permissionList = getPermissionList();
    if (!permissionList.length) return false;

    const requiredPermissions = Array.isArray(permissions) ? permissions : [permissions];
    return requiredPermissions.some(permission => permissionList.includes(permission));
  };
  // 缓存代理以提高性能
  // 注册全局权限检查方法
  app.config.globalProperties.$hadRule = new Proxy(hasPermission, {
    // 创建缓存Map
    cache: new Map(),
    // 代理函数调用
    apply(target, thisArg, args) {
      // 将参数序列化作为缓存key
      const key = JSON.stringify(args);
      // 如果缓存中存在，直接返回缓存结果
      if (this.cache.has(key)) {
        return this.cache.get(key);
      }

      // 如果缓存中不存在，执行原函数并缓存结果
      const result = target.apply(thisArg, args);
      this.cache.set(key, result);
      return result;
    }
  });

  // 注册 src/directive 里的 loading、dropsize、focus 等指令
  setupCustomDirectives(app)
};